@@ -67,6 +67,17 @@ sqlite3.register_converter("pickle", serialiser.loads)
 #sqlite3.PARSE_DECLTYPES = 1
 #sqlite3.PARSE_COLNAMES = 2
 
+import threading
+
+def _get_thread_id():
+    """ returns the current thread id.
+    """
+    if hasattr(threading, 'current_thread'):
+        return threading.current_thread().ident
+    elif hasattr(threading, 'currentThread'):
+        return threading.currentThread().ident
+
+
 
 def deserialise(data):
     return data
@@ -92,6 +103,9 @@ class SQLPickle(object):
         """
 
         self._table_name = table_name
+        self._database = database
+        self._thread_id = _get_thread_id()
+
         self._connect(database=database)
         self._cursor = self._conn.cursor()
         # fields by name, comment out for normal rows.
@@ -110,27 +124,41 @@ class SQLPickle(object):
             pass
             # the table most likely already exists.
 
+        self._conn.commit()
 
-    def save(self, key, value):
+    def _check_connection(self):
+        """
+        """
+        if self._thread_id != _get_thread_id():
+            # reopen the db.
+            self.open(self._table_name, self._database)
+
+
+    def save(self, key, value, commit = True):
         """ save(key, value) adds the value for the given key.
             Overwrites any existing keys value if there is already one.
 
             Note, does not commit automatically.
         """
+        self._check_connection()
+
         # use the highest and fastest protocol available.
         data_string = sqlite3.Binary( serialiser.dumps(value, 2) )
         try:
-            self._conn.execute("insert into %s values (null, ?, ?)" % self._table_name, 
-                               (key, data_string,))
+            q = "insert into %s values (null, ?, ?)" % self._table_name
+            self._conn.execute(q, (key, data_string))
         except sqlite3.IntegrityError:
             # update instead
-            self._conn.execute("update %s set value = ? where key = ?" % self._table_name, 
-                               (data_string, key))
-
+            q = "update %s set value = ? where key = ?" % self._table_name
+            self._conn.execute(q, (data_string, key))
+        if commit:
+            self._conn.commit()
 
     def get(self, key, default = None):
         """ D.get(k[,d]) -> D[k] if k in D, else d.  d defaults to None.
         """
+        self._check_connection()
+
         q = "select * from %s where key = ?" % self._table_name
         arow = self._cursor.execute(q, (key,)).fetchone()
         if arow is None:
@@ -140,6 +168,7 @@ class SQLPickle(object):
     def keys(self):
         """ keys() returns a generator of keys.
         """
+        self._check_connection()
         q = "select key from %s" % self._table_name
         rows = self._cursor.execute(q).fetchall()
         #return (deserialise(row['key']) for row in rows)
@@ -151,19 +180,21 @@ class SQLPickle(object):
             If E lacks .keys() method, does:     for (k, v) in E: D[k] = v
             In either case, this is followed by: for k in F: D[k] = F[k]
         """
+        self._check_connection()
         # TODO: could optimize this to do one SQL call rather than many.
         if hasattr(E, 'keys'):
             for k in E: 
                 #D[k] = E[k]
-                self.save(k, E[k])
+                self.save(k, E[k], False)
         else:
             for (k, v) in E: 
                 #D[k] = v
-                self.save(k, v)
+                self.save(k, v, False)
 
         for k in F: 
-            self.save(k, F[k])
+            self.save(k, F[k], False)
         
+        self._conn.commit()
 
 
 
@@ -171,6 +202,7 @@ class SQLPickle(object):
     def items(self):
         """ items() returns a generator of (key,value) pairs.
         """
+        self._check_connection()
         q = "select * from %s" % self._table_name
         rows = self._cursor.execute(q).fetchall()
         return ((deserialise(row['key']), deserialise(row['value'])) for row in rows)
@@ -181,16 +213,19 @@ class SQLPickle(object):
     def commit(self):
         """commit() commits the changes.
         """
+        self._check_connection()
         self._conn.commit()
 
     def rollback(self):
         """rollback() rolls back the changes.
         """
+        self._check_connection()
         self._conn.rollback()
 
     def close(self):
         """close() closes the database connection.
         """
+        self._check_connection()
         self._conn.close()
 
 
